#include <FreeRTOS.h>
#include <task.h>
#include <portable.h>
#include <string.h>

#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/exti.h>
//#include <libopencm3/stm32/usart.h>
#include <libopencm3/stm32/spi.h>
#include <libopencm3/cm3/nvic.h>

//#include "lib/mini-printf.h"
#include "lib/gpio_utils.h"
#include "lib/ssd1306.h"

#ifdef __GNUC__
#define UNUSED(x) UNUSED_##x __attribute__((__unused__))
#else
#define UNUSED(x) UNUSED_##x
#endif

#define DDS_REF ((float)125e6)

//add stack-overflow hook
extern void vApplicationStackOverflowHook(xTaskHandle *pxTask, signed portCHAR *pcTaskName);
void task_draw(void *UNUSED(args));

// our tasks
void task_onboardled(void *args);

/*
 * Controlliamo uno schermo OLED con protocollo SPI
 */

struct ssd1306 oled1  = {SPI2, GPIOB, GPIO13, GPIO15, GPIO12, GPIO10, GPIO11};
struct ssd1306 oled2 = {SPI1, GPIOA, GPIO5,  GPIO7,  GPIO4,  GPIO12, GPIO11};

static void gpio_setup(void)
{
	// built-in LED
	rcc_periph_clock_enable(RCC_GPIOC);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_PUSHPULL, GPIO13);

    // OLED1
	rcc_periph_clock_enable(RCC_GPIOB);
	rcc_periph_clock_enable(RCC_SPI2);
    oled_setup(&oled1);
    // OLED2
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_SPI1);
    oled_setup(&oled2);
}

int main(void)
{
    rcc_clock_setup_in_hse_8mhz_out_72mhz();
	gpio_setup();

	xTaskCreate(task_draw, "DRAW", 500, NULL, configMAX_PRIORITIES - 1,
	            NULL);
	xTaskCreate(task_onboardled, "ONBOARD", 50, NULL, configMAX_PRIORITIES - 1,
	            NULL);
	vTaskStartScheduler();
	for (;;) {
	}

	return 0;
}

void task_draw(void *UNUSED(args))
{
    uint8_t c = 0;
    oled_init(&oled1);
    oled_init(&oled2);

    vTaskDelay(pdMS_TO_TICKS(10));
    while(1) {
        c++;
        uint8_t graph[128*64/8];
        for(int i=0; i<128*64/8; i++) {
            graph[i] = c;
        }
        oled_draw(&oled1, graph);
        oled_draw(&oled2, graph);
		vTaskDelay(pdMS_TO_TICKS(40));
    }
}

void task_onboardled(void *UNUSED(args))
{
	while (1) {
        gpio_clear(GPIOC, GPIO13); /* LED on */
        vTaskDelay(pdMS_TO_TICKS(300));

		gpio_set(GPIOC, GPIO13); /* LED off */
		vTaskDelay(pdMS_TO_TICKS(200));
	}
}

// three "small flashes" (an S in Morse code) on onboard led
//
// questa funzione viene chiamata in caso di stackoverflow su uno dei task; e'
// quindi utile metterci un "segnale" riconoscibile per debug. In questo caso 3
// flash rapidi.
//
// siccome "tarare" la dimensione dello stack necessario e' difficile, con
// questa funzione si puo' andare "a tentoni" e ridurre la dimensione finche'
// non si ottengono crash
//
// in questa funzione, a quanto pare (ma non ho trovato doc) non si possono
// usare cose come vTaskDelay. Siamo quindi privi di RTOS e per fare gli sleep
// dobbiamo fare i for "all'antica"

void vApplicationStackOverflowHook(xTaskHandle *UNUSED(pxTask),
        signed portCHAR *UNUSED(pcTaskName))
{
    int i;
    for (;;) {
        for (char c = 0; c < 3; c++) {
            gpio_clear(GPIOC, GPIO13); /* LED on */
            for (i = 0; i < 1000000; i++)   /* small delay */
                __asm__("nop");

            gpio_set(GPIOC, GPIO13);   /* LED off */
            for (i = 0; i < 3000000; i++) { /* a bit more delay */
                __asm__("nop");
            }
        }
        for (i = 0; i < 10000000; i++) {    /* long delay */
            __asm__("nop");
        }
    }
}

// vim: set fdm=marker:
